module Ncode
  def ncode(str)
    delascii = str.gsub(/[\x00-\x7f]/, '')

    # strict check start
    if str =~ /[\x00-\x06\x7f\xff]/ then # binary
      return "BINARY"
    elsif str =~ /\x1b\x24\x40|\x1b\x24\x42|\x1b\x24\x28\x44/ then # JIS
      return "ISO-2022-JP"
    elsif delascii.length == 0 then
      return "ASCII"
    elsif str =~ /[\xa1-\xfe][\xfd-\xfe]/ then # binyu-table
      return "EUC-JP"
    end
    # strict check end

    deleuc = str.gsub(/[\xa1-\xfd][\xa1-\xfd]/, '')
    delutf = str.gsub(/[\xc0-\xdf][\x80-\xbf]|[\xe0-\xef][\x80-\xbf][\x80-\xbf]/, '')
    delsjis = str.gsub(/[\x81-\x9f\xe0-\xfc][\x40-\x7e\x80-\xfc]/, '')

    if deleuc.length < delsjis.length and 
        deleuc.length < delutf.length then
      return "EUC-JP"
    elsif delutf.length < deleuc.length and
        delutf.length < delsjis.length then
      return "UTF-8"
    elsif delsjis.length < delutf.length and
        delsjis.length < deleuc.length then
      return "Shift_JIS"
    else
      return "UNKOWN"
    end
  end
  module_function :ncode
end

class String
  def ncode(str)
    Ncode::ncode(self)
  end
end
