#!/usr/bin/ruby

def call_usage()
  print <<END_OF_USAGE
usage: nkc [-e|-j|-s|-w] [any options] file1 file2 file3....

-e      convert to EUC-JP
-j      convert to ISO-2022-JP(JIS)
-s      convert to Shift_JIS
-w      convert to UTF-8
-m      convert to CR+LF(DOS) newline code
-u      convert to LF(UNIX) newline code
-M      convert to CR(MAC) newline code
--help  show this message.
END_OF_USAGE
exit
end

def call_error(msg, reason)
  print msg, ": ", reason, "\n"
  call_usage()
end

def read_args(*opts)
  for i in 0 ... opts.length
    case opts[i]
    when '-s'
      to = 'Shift_JIS'
    when '-e'
      to = 'EUC-JP'
    when '-j'
      to = 'ISO-2022-JP'
    when '-w'
      to = 'UTF-8'
    when '-m'
      newline = "\r\n" # CR+LF
    when '-u'
      newline = "\n"   # LF
    when '-ma'
      newline = "\r"   # CR
    when '-M'
      newline = "\r"   # CR
    when /-h|-H|--help/
      call_usage()
    else
      call_error('Invalid option', opts[i])
    end
  end
  return to, newline
end


##################################### main ####################################
require 'libncode.rb'
require 'iconv'

# separate options and files
opts = Array.new()
files = Array.new()

for i in 0 ... ARGV.length
  if ARGV[i] =~ /^-/ then
    opts.push(ARGV[i])
  elsif
    files.push(ARGV[i])
  end
end
# parse options
to, newline = read_args(*opts)

# No file
if files.length == 0 then
  call_usage()
end

# main
for i in 0 ... files.length
  # file open
  begin
    f = open(files[i], 'r+')
  rescue
    raise
  end

  body = f.read()
  from = Ncode::ncode(body)

  print files[i], ": "
  if from =~ /Shift_JIS|EUC-JP|ISO-2022-JP|UTF-8/ and to != nil then
    c = Iconv.new(to, from)
    body = c.iconv(body)
    c.close
    print from, " to ", to
  else
    print "Cannot convert ", from, " to ", to
  end

  if newline then
    case newline
    when "\r\n"
      print " (CR+LF)"
    when "\r"
      print " (CR)"
    when "\n"
      print " (LF)"
    end      
    body = body.gsub(/\r\n|\r|\n/, newline)
  end

  print "\n"

  f.truncate(0)
  f.rewind
  f.write body
  f.close
end
